
/**
 * @file SerialCmd.h
 * @author Ollo
 * @brief 
 * @version 0.1
 * @date 2024-12-06
 * 
 * @copyright Copyright (c) 2024
 * 
 */
#include "SerialCmd.h"
#include "ColorUtil.h"
#include <Arduino.h>


// Serial fields
#define CMD_MAX     128

#define CMD_INDEX   5 /**< fiveth character defindes the command */

static char myCmd[CMD_MAX];

/******************************************************************************
 *                            LOCAL FUNCTIONS
 ******************************************************************************/

static unsigned int readLowNipple(int i, unsigned int startOffset);

static unsigned int readHighNipple(int i, unsigned int startOffset);


/******************************************************************************
 *                           EXTERNAL FUNCTIONS
 ******************************************************************************/

/**
 * @brief 
 * number of read bytes
 * @return int 
 */
int SerialCmd_readFromSerial (void)
{
  //read from the serial buffer and flush
  int inputSize = Serial.available();
  
  //give serial a chance to receive all bytes
  if(inputSize > 0){
    delay(100);
    inputSize = Serial.available();
  }
  
  if(inputSize > 0 && inputSize < CMD_MAX)
  {
    Serial.print("inputSize: ");
    Serial.print(inputSize);
    for (int i = 0; i < inputSize; i++){
      myCmd[i] = Serial.read();
    }
  }
  else if(inputSize >= CMD_MAX)
  {
    for (int i = 0; i < inputSize; i++){
      Serial.read();
    }
    Serial.print("too much data, flush");
  }
  return inputSize;
}

/**
 * @brief 
 * clear the cmd array
 */
void SerialCmd_clearCmdArray(void)
{
  for (int i = 0; i < CMD_MAX; i++)
  {
    myCmd[i] = '\0';
  }
}

/**
 * @brief 
 * check if command has the required prefix
 * @return int 
 */
int SerialCmd_checkCmdArrayForPrefix (void)
{
   if (myCmd[0] == 'o'
     && myCmd[1] == 'l'
     && myCmd[2] == 'l'
     && myCmd[3] == 'p'
     && myCmd[4] == 'e')
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/**
 * @brief 
 * Get type, see different commands in header file
 * 
 * @return char 
 */
char SerialCmd_type(void)
{
    if (SerialCmd_checkCmdArrayForPrefix())
    {
        return myCmd[CMD_INDEX];
    }
    else
    {
        return '\0';
    }
}

/**
 * @brief Color
 * Extract color of the following six bytes
 * @param startOffset 
 * @return uint32_t Color as red, green and blue, @see Color
 */
uint32_t SerialCmd_parseColor(unsigned int startOffset)
{
  unsigned int colorParts[3]; /* 0: red, 1: green, 2: blue */

  if (startOffset > (CMD_MAX - (sizeof(colorParts) * 2)))
  {
    return 0;
  }

  for (int i=0; i < (2*3); i+= 2)
  {
    colorParts[i / 2] = readHighNipple(i, startOffset);
    if (colorParts[i/2] < 0)
    {
        Serial.print("calcErr:"); 
        Serial.println(colorParts[i/2]);
    }
    colorParts[i / 2] += readLowNipple(i, startOffset);
  }
  int r = colorParts[0]; /* fill red */
  int g = colorParts[1]; /* fill green */
  int b = colorParts[2]; /* fill blue */

  return Color(r, g, b);
}

static unsigned int readLowNipple(int i, unsigned int startOffset)
{
  return ((myCmd[i + startOffset + 1] <= '9') ? (myCmd[i + startOffset + 1] - '0') : (((myCmd[i + startOffset + 1] - 'A') + 10)));
}

static unsigned int readHighNipple(int i, unsigned int startOffset)
{
  return (myCmd[i + startOffset] <= '9') ? ((myCmd[i + startOffset] - '0') << 4) : (((myCmd[i + startOffset] - 'A') + 10) << 4);
}

/**
 * @brief 
 * Parse the following two bytes as hex values
 * @param startOffset 
 * @return int number (0-255) or negative values on failure
 */
int SerialCmd_parseNumber(unsigned int startOffset)
{

  if (startOffset > (CMD_MAX - 2))
  {
    return -1;
  }
  else
  {
    int value=0;
    value = readHighNipple(0, startOffset);
    value += readLowNipple(0, startOffset);
    return value;
  }
}

void SerialCmd_help(void)
{
  Serial.println(SERIALCMD_COMMAND_HELP);
  Serial.println(F("help: " SERIALCMD_COMMAND_HELP)); 
  Serial.println(F("leds: " SERIALCMD_COMMAND_SETLED  "followed by rrggbb (as hex)"));
  Serial.println(F("manual fan control: " SERIALCMD_COMMAND_FAN " followed by xx (speed as hex)"));
  Serial.println(F("automatic fan: " SERIALCMD_COMMAND_FAN " followed by xx (speed as hex)"));
}
